with Ada.Characters.Handling;
with Ada.Text_IO, Ada.Command_Line, Ada.Directories, Ada.Calendar;
use  Ada.Text_IO;

procedure Mondrian is

  --====================================================================
  -- Autor     Christoph Grein
  -- Version   1.2
  -- Datum     8 Mai 2023
  --====================================================================
  -- Quellkode des Mondrian-Compilers
  -- Generierung des Ampelschaltplans aus der Farbkodierung Name.csv.
  -- (Statt eines Kommas erzeugt mein Excel einen Strichpunkt.)
  -- Erzeugt werden die Spezifikation und der Rumpf Ampel.Name in der
  -- GNAT-Namenskonvention (in Kleinbuchstaben).
  --====================================================================
  -- Geschichte
  -- Autor  Version  Datum    Grund der nderung
  --  C.G.    1.2  08.05.2023 berprfung auf Aktualitt
  --  C.G.    1.1  17.04.2023 Dauer als Positive (war Duration)
  --  C.G.    1.0  13.04.2023 Erzeugt fr Ada-Tasking-Kurs
  --====================================================================

  function Strichpunkt (Zeile: in String) return Positive is
    pragma Warnings (Off);
  begin
    for Z in Zeile'Range loop
      if Zeile (Z) = ';' then
        return Z;
      end if;
    end loop;
  end Strichpunkt;

  -- Leider habe ich keine Mglichkeit gefunden, aus der Excel-Datei programmatisch die CSV-Datei zu erzeugen.
  -- Das muss leider hndisch erfolgen.
  CSV_Datei : constant String := Ada.Command_Line.Argument (1);                 -- C:\Path\Ampel\Farbkodierung\Modell_Kreuzung.csv
  Farb_Datei: constant String := CSV_Datei (1 .. CSV_Datei'Last - 3) & "xlsx";  -- C:\Path\Ampel\Farbkodierung\Modell_Kreuzung.xlsx
  CSV_Name  : constant String := Ada.Directories.Simple_Name (CSV_Datei);                                   -- Modell_Kreuzung.csv
  Basis_Name: constant String := Ada.Directories.Base_Name   (CSV_Name);                                    -- Modell_Kreuzung
  Ada_Name  : constant String := "Ampel." & Basis_Name;                                                     -- Ampel.Modell_Kreuzung
  ads_Datei : constant String := "ampel-" & Ada.Characters.Handling.To_Lower (Basis_Name) & ".ads";         -- ampel-modell_kreuzung.ads
  adb_Datei : constant String := "ampel-" & Ada.Characters.Handling.To_Lower (Basis_Name) & ".adb";         -- ampel-modell_kreuzung.adb
  -- Das erzeugte Paket Ada_Name liegt in einem anderen Verzeichnis als die Farb-Datei.

  CSV_Veraltet: exception;

  function Ist_Aktuell return Boolean is
    -- Die Farb-Datei darf nicht neuer als die CSV-Datei sein.
    -- Dann ist auch das Ada-Paket (so es schon existiert), veraltet.
    use Ada.Directories, Ada.Calendar;
  begin
    if Modification_Time (CSV_Datei) <= Modification_Time (Farb_Datei) then
      raise CSV_Veraltet;
    end if;
    return Exists (ads_Datei) and then
           Modification_Time (Farb_Datei) <= Modification_Time (ads_Datei);
  end Ist_Aktuell;

  Quelle, Spez, Rumpf: File_Type;  -- die Ausgangsdatei und die Erzeugten

  procedure Kopf is separate;
  procedure Schaltplan (Anzahl_Phasen: in Positive) is separate;

begin

  -- Put_Line (Farb_Datei);
  -- Put_Line (CSV_Datei);
  -- Put_Line (CSV_Name);
  -- Put_Line (Basis_Name);
  -- Put_Line (Ada_Name);
  -- Put_Line (ads_Datei);

  if Ist_Aktuell then
    Put_Line ("Paket " & Ada_Name & " ist aktuell.");
    return;
  end if;

  Put_Line ("Erzeuge Paket " & Ada_Name & '.');

  Open   (Quelle, In_File , CSV_Datei);
  Create (Spez  , Out_File, ads_Datei);
  Create (Rumpf , Out_File, adb_Datei);

  Set_Input  (Quelle);

  Kopf;

  Lies_Spuren:
  declare
    Zeile: constant String := Get_Line;
    Anfang   : Positive := Strichpunkt (Zeile);                           -- berspringe t
    Ende     : Positive := Strichpunkt (Zeile (Anfang+1 .. Zeile'Last));  -- berspringe delta_t
    Folgender: Boolean  := False;  -- Sonderbehandlung fr ersten Bezeichner
  begin
    Put (Spez, "  type Ampelspur is (");
    while Ende < Zeile'Last loop
      Anfang := Ende;
      Ende   := Strichpunkt (Zeile (Anfang+1 .. Zeile'Last));
      Put (Spez, (if Folgender then ", " else "") & "Sp_" & Zeile (Anfang+1 .. Ende-1));  -- mach gltigen Bezeichner
      Folgender := True;
      Ende := Ende + 1;  -- berspringe leeres Feld
    end loop;
    Put_Line (Spez, ");");
  end Lies_Spuren;

  New_Line (Spez);

  Zhle_Zykluszeilen:
  declare
    Anzahl: Natural := 0;
  begin
    while not End_Of_File loop
      declare
        Zeile: constant String := Get_Line;
      begin
        Anzahl := Anzahl + 1;
      end;
    end loop;
    Put_Line (Spez, "  type Ampelzyklus is mod" & Anzahl'Image & ";");
    Reset (Quelle);
    Schaltplan (Anzahl);
  end Zhle_Zykluszeilen;

  New_Line (Spez);

  Put_Line (Spez , "  function Dauer (Zyklus: Ampelzyklus) return Positive;");
  Put_Line (Rumpf, "  function Dauer (Zyklus: Ampelzyklus) return Positive is");
  Put_Line (Rumpf, "  begin");
  Put_Line (Rumpf, "    return Schaltplan (Zyklus).Dauer;");
  Put_Line (Rumpf, "  end Dauer;");

  New_Line (Rumpf);

  Put_Line (Spez , "  function Farbe (Zyklus: Ampelzyklus; Spur: Ampelspur) return Ampelfarbe;");
  Put_Line (Rumpf, "  function Farbe (Zyklus: Ampelzyklus; Spur: Ampelspur) return Ampelfarbe is");
  Put_Line (Rumpf, "  begin");
  Put_Line (Rumpf, "    return Schaltplan (Zyklus).Farbe (Spur);");
  Put_Line (Rumpf, "  end Farbe;");

  New_Line (Spez);
  New_Line (Rumpf);

  Put_Line (Spez , "end " & Ada_Name & ';');
  Put_Line (Rumpf, "end " & Ada_Name & ';');

  Close (Quelle);  -- Delete
  Close (Spez);
  Close (Rumpf);

exception
  when CSV_Veraltet =>  -- Das ist eine echte Ausnahme! Die bersetzung des Ampel-Programms kann nicht ausgefhrt werden.

    Ada.Directories.Delete_File (CSV_Datei);

    if Ada.Directories.Exists (ads_Datei) then
      Ada.Directories.Delete_File (ads_Datei);
    end if;

    Put_Line (CSV_Name & " (und das Paket " & Ada_Name & ", so es existiert) sind veraltet und mssen neu erzeugt werden.");

end Mondrian;
