package body Ampel.Steuerung is

  --====================================================================
  -- Autor     Christoph Grein
  -- Version   1.2
  -- Datum     19 April 2023
  --====================================================================
  --
  --====================================================================
  -- Geschichte
  -- Autor  Version  Datum    Grund der nderung
  --  C.G.    1.2  19.04.2023 Farben Gelb_Dunkel und Dunkel hinzugefgt
  --  C.G.    1.1  17.04.2023 Dauer als Positive (war Duration)
  --  C.G.    1.0  16.04.2023 Kontrollpanel implementiert
  --  C.G.    0.0  13.04.2023 Erzeugt fr Ada-Tasking-Kurs
  --                          Kontrollpanel fehlt noch
  --====================================================================

  subtype Einschaltphase is Prozessorzustand range Aus .. Rot;

  Zustand: Prozessorzustand := Aus;

  function Ampelzustand return Prozessorzustand is (Zustand);

  ----------------------

  task body Prozessor is

    Zyklus  : Ampelzyklus := 0;
    Sekunden: Natural     := 0;     -- Rest der Dauer des aktuellen Zyklus
    nderung: Boolean     := True;  -- der Zustand hat sich gendert

    type Gelb_Dunkel_Sonderfall is mod 3;
    Sonder_GD: Gelb_Dunkel_Sonderfall := 0;

    procedure Set_Ampel (Phase: Einschaltphase) is
      -- Alle Ampeln zeigen synchron dasselbe Bild.
    begin
      --parallel    Ada 2022
      for Spur in Ampelspur loop
        case Phase is
          when Aus      => Set_Ampel (Spur);                  -- alle Ampeln aus
          when Blinkend => Set_Ampel (Spur, Gelb => True);    -- alle Ampeln blinken
          when Rot      => if nderung then                   -- setze alle Ampeln
                             Set_Ampel (Spur, Gelb => True);  --   erst alle auf gelb (fix)
                           else                               --   beim zweiten Aufruf
                             Set_Ampel (Spur, Rot  => True);  --   dann auf rot
                           end if;
        end case;
      end loop;
    end Set_Ampel;

  begin

    Lebenslang:
    loop

      -- Einschaltphase
      select
        accept Ein;
        case Zustand is
          when Einschaltphase => Zustand  := Prozessorzustand'Succ (@);
          when Normal         => Zustand  := Blinkend;  -- alle Ampeln fallen auf Anforderung von
                                 Zyklus   := 0;         -- auen zurck in die Einschaltphase
                                 Sekunden := 0;
        end case;
        nderung := True;
      or
        accept Aus;
        Zustand  := Aus;
        Zyklus   := 0;
        Sekunden := 0;
        nderung := True;
      or
        accept Ende;
        exit Lebenslang;
      else
        null;
      end select;

      -- Zustandsnderung
      case Zustand is
        when Aus      => if nderung then
                           Set_Ampel (Aus);
                           nderung := False;
                         end if;
        when Blinkend => Set_Ampel (Blinkend);  delay 0.5;
                         Set_Ampel (Aus);       delay 0.5;
        when Rot      => if nderung then                   -- alle fr kurze Zeit
                           Set_Ampel (Rot);     delay 4.0;  --   auf gelb (kein Blinken), Parameter Rot ist richtig!
                           nderung := False;               -- dann
                           Set_Ampel (Rot);                 --   auf rot
                         end if;
        when Normal   => --parallel    Ada 2022
                         for Spur in Ampelspur loop
                           case Farbe (Zyklus, Spur) is
                             when Rot         => Set_Ampel (Spur, Rot  => True);
                             when Rot_Gelb    => Set_Ampel (Spur, Rot  => True, Gelb => True);
                             when Grn        => Set_Ampel (Spur, Grn => True);
                             when Gelb        => Set_Ampel (Spur, Gelb => True);
                             when Dunkel      => Set_Ampel (Spur);
                             when Gelb_Dunkel => case Sonder_GD is
                                                   when 0 => Set_Ampel (Spur, Gelb => True);  -- gelb und
                                                   when 1 => Set_Ampel (Spur);                -- dunkel blinken
                                                   when 2 => null;  -- kommt nicht vor
                                                 end case;
                                                 Sonder_GD := Sonder_GD + 1;
                           end case;
                         end loop;
                         -- Die Dauer wird in Schritten von 1s abgebaut, damit Eingnge
                         -- schnell bedient werden.
                         if Sekunden = 0 then           -- die erste Sekunde im Zyklus
                           Sekunden := Dauer (Zyklus);  -- lade die Dauer
                         end if;
                         case Sonder_GD is  -- Dieses Blinken ist nicht das der Einschaltphase!
                           when 0 => delay 1.0;           -- alle anderen Farben
                                     Sekunden  := @ - 1;  -- die erste Sekunde ist verbraucht
                           when 1 => delay 0.5;           -- Gelb
                           when 2 => delay 0.5;           --      Dunkel blinken
                                     Sonder_GD := @ + 1;
                                     Sekunden  := @ - 1;  -- die erste Sekunde ist verbraucht
                         end case;
                         if Sekunden = 0 then           -- das war die letzte Sekunde des Zyklus
                           Zyklus := Zyklus + 1;        -- lade neuen Zyklus
                         end if;
      end case;

    end loop Lebenslang;

  end Prozessor;

end Ampel.Steuerung;
