with Waehrung;
use  Waehrung;

package Fahrkartenautomat is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      9 April 2014
  --====================================================================
  -- Das Paket dient der Simulation eines Fahrkartenautomaten.
  -- die Bedienung erfolgt ber ein Kommandozeilen-Terminal (in Ada-
  -- Nomenklatur: Standard_Input fr Eingabe der Fahrkartenwahl und der
  -- Geldstcke; Standard_Output fr Bedienungsanleitung).
  -- Da keine grafische Oberflche zur Verfgung steht, die gewisse
  -- Eingaben verbietet, werden ungltige Eingaben durch entsprechende
  -- Meldungen angezeigt.
  --====================================================================
  -- History
  -- Author Version   Date    Reason for change
  --  C.G.    1.0  09.04.2014 Created for Ada tasking course
  --====================================================================

  subtype Akzeptiertes_Geldstueck is Geldstueck range Cent_10 .. Euro_10;

  type Fahrkartenoption is (Keine, Kurzstrecke, Innenraum, Gesamtgebiet, Tageskarte);
  subtype Fahrkarte is Fahrkartenoption range Kurzstrecke .. Tageskarte;

  -- 1. Fahrkarte whlen.
  --    Kann nicht wiederholt werden vor Beendigung des gesamten Vorgangs
  --    (bei Wiederholung wird "Taste auer Funktion" ausgegeben).
  --    Vor Wahl einer Karte ist die Geldeingabe unmglich.
  procedure Waehle (F: Fahrkarte);

  -- 2. Geld einwerfen.
  --    Kann nicht ausgefhrt werden vor Fahrkartenwahl und nach Beendigung
  --    des Vorgangs.
  --    Der Vorgang wird beendigt, sobald gengend Geld eingeworfen ist oder
  --    wenn Abbruch gewhlt wird. Auerdem wird der Vorgang abgebrochen,
  --    wenn zu lange keine Eingabe erfolgt. Das bedeutet, dass die Blockade
  --    des Aufrufers von Ausgabe beendet wird.
  --    Ergebnis ist True, wenn Geld akzeptiert wurde, False sonst (dann
  --    darf das Vermgen des Kunden nicht um den Betrag vermindert werden.)
  --    (Bei Aufruf vor Fahrkartenwahl wird "Geldeingabeschlitz geschlossen"
  --    ausgegeben.)
  procedure Geldeingabe (S: Akzeptiertes_Geldstueck; Ergebnis: out Boolean);
  procedure Abbruch;

  -- 3. Ausgabe der Fahrkarte.
  --    Ende des Vorgangs.
  --    Bei Abbruch wird das eingeworfene Geld zurckerstattet und keine
  --    Fahrkarte ausgegeben.
  --    (Der Aufrufer wird solange blockiert, bis die Ausgabe erfolgt.)
  procedure Ausgabe (Wechselgeld   : out Stueckelung;
                     eine_Fahrkarte: out Fahrkartenoption);

end Fahrkartenautomat;
