------------------------------------------------------------------------
-- This software is provided as is in the hope that it might be found
-- useful.
-- You may use and modify it freely provided you keep this copyright
-- notice unchanged and mark modifications appropriately.
--
-- Bug reports and proposals for improvements are welcome. Please send
-- them to the eMail address below.
--
-- Christoph Karl Walter Grein
-- Hauptstr. 42
-- D-86926 Greifenberg
-- Germany
--
-- eMail: Christ-Usch.Grein@T-Online.de
--
-- Copyright (c) 2022 Christoph Karl Walter Grein
------------------------------------------------------------------------

with Ada.Text_IO;

with Monte_Carlo;

package body Flughafen is

  --====================================================================
  -- Autor     Christoph Grein
  -- Version   1.0
  -- Datum     29 December 2022
  --====================================================================
  -- Definition der Prozessrmpfe.
  --====================================================================
  -- Geschichte
  -- Autor  Version   Datum   Grund der nderung
  --  C.G.    1.0  29.12.2022 Erzeugt fr Ada-Kurs Nebenlufige Prozesse
  --====================================================================

  -- Nach ffnung ist der Schalter frei, einen Fluggast zu empfangen.
  -- Dafr schickt er die Nachricht "Bin Frei" an die groe Warte-
  -- schlange und wartet auf das Fluggastrendezvous, das sofort beginnen
  -- kann, da nach Konstruktion keine Warteschlange davor entstehen kann.
  task body Schalter is
    Wer: Fluggast.Name;
  begin
    delay Monte_Carlo.Schalterffnung;
    loop
      Ada.Text_IO.Put_Line ("Schalter" & Index'Image & " ist frei.");
      Warteschlange.Bin_Frei (Index);
      accept Checkin (Name: Fluggast.Name; Gepck: Duration) do
        delay Gepck;  -- Abfertigung dauert
        Wer := Name;
      end Checkin;
      Ada.Text_IO.Put_Line ("Fluggast" & Wer'Image & " am Schalter" & Index'Image & " ist abgefertigt.");
    end loop;
    -- Der Prozess wird nie beendet.
  end Schalter;

  -- Der Anstellen-Eingang ist gesperrt (Wchter Frei), bis einer der
  -- Check-In-Schalter sich frei meldet. (Eine Liste der freien Schalter
  -- wird gefhrt. Der Wchter wird jedesmal ausgewertet, wenn die
  -- Select-Anweisung erreicht wird.)
  task body Warteschlange is

    Freie_Schalter    : array (Schalterindex) of Boolean := (others => False);
    Gewhlter_Schalter: Schalterindex;

    function Frei return Boolean is
    begin
      for Index in Schalterindex loop
        if Freie_Schalter (Index) then
          Gewhlter_Schalter := Index;
          return True;
        end if;
      end loop;
      return False;
    end Frei;

  begin

    loop

      select
        when Frei =>
          accept Anstellen (Name: Fluggast.Name; Wohin: out Schalterindex) do
            Ada.Text_IO.Put_Line ("Fluggast" & Name'Image & " wird zu Schalter" & Gewhlter_Schalter'Image & " geschickt.");
            Wohin := Gewhlter_Schalter;
          end Anstellen;
          Freie_Schalter (Gewhlter_Schalter) := False;
      or
        accept Bin_Frei (Index: Schalterindex) do
          Freie_Schalter (Index) := True;
          Ada.Text_IO.Put_Line ("Schalter" & Index'Image & " ist freigegeben.");
        end Bin_Frei;
      end select;

    end loop;

    -- Der Prozess wird nie beendet.

  end Warteschlange;

end Flughafen;
