------------------------------------------------------------------------
-- This software is provided as is in the hope that it might be found
-- useful.
-- You may use and modify it freely provided you keep this copyright
-- notice unchanged and mark modifications appropriately.
--
-- Bug reports and proposals for improvements are welcome. Please send
-- them to the eMail address below.
--
-- Christoph Karl Walter Grein
-- Hauptstr. 42
-- D-86926 Greifenberg
-- Germany
--
-- eMail: Christ-Usch.Grein@T-Online.de
--
-- Copyright (c) 2022 Christoph Karl Walter Grein
------------------------------------------------------------------------

with Prozesstaufe;
with Fluggast;

package Flughafen is

  --====================================================================
  -- Autor     Christoph Grein
  -- Version   1.0
  -- Datum     29 December 2022
  --====================================================================
  -- Definition des Prozesses eines Check-In-Schalters mit Index als
  -- Identitt und der Reihe aller Schalter.
  -- Ma fr die Dauer des Check-In ist das Gepck.
  -- Definition der groen Warteschlange mit Eingngen:
  -- Der Fluggast stellt sich an und erhlt, sobald er der erste in der
  -- Warteschlange ist und ein Schalter sich frei meldet, die
  -- entsprechende Adresse zum Check-In.
  -- Die Initialisierung mit Iterator (iterated_component_association)
  -- ist Ada 2022.
  --====================================================================
  -- Geschichte
  -- Autor  Version   Datum   Grund der nderung
  --  C.G.    1.0  29.12.2022 Erzeugt fr Ada-Kurs Nebenlufige Prozesse
  --====================================================================

  type Schalterindex is range 1 .. 5;

  task type Schalter (Index: Schalterindex := Schalterindex'First) is
    entry Checkin (Name: Fluggast.Name; Gepck: Duration);
  end Schalter;

  function Neuer_Schalter is new Prozesstaufe (Schalterindex, Schalter);

  Schalterreihe: constant array (Schalterindex) of Schalter := (for Index in Schalterindex => Neuer_Schalter (Index));

  task Warteschlange is
    entry Anstellen (Name : Fluggast.Name; Wohin: out Schalterindex);
    entry Bin_Frei  (Index: Schalterindex);
  end Warteschlange;

end Flughafen;
