------------------------------------------------------------------------
-- This software is provided as is in the hope that it might be found
-- useful.
-- You may use and modify it freely provided you keep this copyright
-- notice unchanged and mark modifications appropriately.
--
-- Bug reports and proposals for improvements are welcome. Please send
-- them to the eMail address below.
--
-- Christoph Karl Walter Grein
-- Hauptstr. 42
-- D-86926 Greifenberg
-- Germany
--
-- eMail: Christ-Usch.Grein@T-Online.de
--
-- Copyright (c) 2013, 2023 Christoph Karl Walter Grein
------------------------------------------------------------------------

with Ada.Finalization;
-- with Ada.Text_IO;
-- use  Ada.Text_IO;

package body Lock_Object is

  --====================================================================
  -- Author    Christoph Grein
  -- Version   1.0
  -- Date      4 May 2013
  --====================================================================
  -- The lock object, a semaphore, is wrapped in a limited controlled
  -- object so that initialization does the locking and finalization
  -- does the unlocking (that way you don't have to worry about abort or
  -- exceptions screwing up your locking).
  --====================================================================
  -- History:
  -- Author Version   Date    Reason for Change
  --  C.G.   1.0  04.05.2013  Created for Ada course
  --====================================================================

  protected Semaphore is
    entry Lock;
    procedure Unlock;
  private
    Locked: Boolean := False;
  end Semaphore;

  protected body Semaphore is
    entry Lock when not Locked is
    begin
      Locked := True;
    end Lock;
    procedure Unlock is
    begin
      Locked := False;
    end Unlock;
  end Semaphore;

  type Locker is new Ada.Finalization.Limited_Controlled with null record;

  procedure Initialize (S: in out Locker);
  procedure Finalize   (S: in out Locker);

  procedure Initialize (S: in out Locker) is
  begin
    -- Put_Line ("Initializing");
    Semaphore.Lock;
    -- Put_Line ("Initialized");
  end Initialize;

  procedure Finalize (S: in out Locker) is
  begin
    -- Put_Line ("Finalizing");
    Semaphore.Unlock;
    -- Put_Line ("Finalized");
  end Finalize;

  procedure Call (X: Item) is
    Lock: Locker;
  begin
    Callee (X);
  end Call;

end Lock_Object;
